% MATLAB program to convert a file containing I 
% and Q samples into a binary file that is accepted
% by the Com-8001 ComBlock module. This program will
% only work with the Com-2001 module and the only
% acceptable output width option is 20 bits.
% The input file must have two columns separated 
% by a tab. The first column must contain the I 
% samples in the range [0  1,023] and the 
% second column must contain the Q samples in 
% the same range. The format for the samples is 
% regular readable ASCII (i.e. double, binary
% or anything else will not work).


% Replace 'input_file_name_including_full_path'
% with the full path and file name for the 
% I and the Q samples are stored.
% For example:
% iAndQSamples = load('c:\My Documents\iAndQData.txt');
iAndQSamples = load('input_file_name_including_full_path');


% Split the read matrix up into two column vector, 
% the first one holding the I data and the second 
% holding the Q data.
iSamples = iAndQSamples(:, 1);
qSamples = iAndQSamples(:, 2);


% Find out how many samples there are. The number of I
% samples is the same as the number of Q samples.
noOfISamples = length(iSamples);


% Make sure all the samples are in the 
% valid range, [0  1,023].
continue = 1;
for i = 1:noOfISamples 
   if (iSamples(i) > 1023) 
      disp('Too big I value in input file.')
      disp('No file will be created.')
		continue = 0;
   elseif (iSamples(i) < 0) 
   	disp('Too small I value in input file.')   
      disp('No file will be created.')
		continue = 0;
   end
   if (qSamples(i) > 1023) 
   	disp('Too big Q value in input file.')   
      disp('No file will be created.')
		continue = 0;
   elseif (qSamples(i) < 0) 
   	disp('Too small Q value in input file.')   
      disp('No file will be created.')
		continue = 0;
   end
end


% Convert the I and the Q samples to binary strings
if (continue == 1)
	for i = 1:noOfISamples
     	iSamplesBin(i,1:10) = dec2bin(iSamples(i), 10);
     	qSamplesBin(i,1:10) = dec2bin(qSamples(i), 10);
	end
end


% Flip the bits of the I samples and the Q samples.
% (MSb on the Com-2001 input connector corresponds 
% to the LSb on Com-8001 output connector.
iSamplesBinFlipped = fliplr(iSamplesBin);
qSamplesBinFlipped = fliplr(qSamplesBin);


% Create a one-dimensional array consisting of the Q 
% and I samples interleaved. Note that we add the 
% Q sample first followed by an I samples because 
% the I samples appear on bits 19-10 and the Q samples 
% appear on bits 9-0 on the output connector of the 
% Com-8001 module.
if (continue == 1)
	samples1Dim = strcat(qSamplesBinFlipped(1,:), iSamplesBinFlipped(1,:));
	for i = 2:noOfISamples
		samples1Dim = strcat(samples1Dim, qSamplesBinFlipped(i,:));
		samples1Dim = strcat(samples1Dim, iSamplesBinFlipped(i,:));
	end
end	


% Find out how many bits we can convert into bytes. If the total number
% of bits is not a multiple of 8 then we will not be able to include
% all bits.
if (continue == 1)
   noOfBytesNeeded = ceil(length(samples1Dim)/8);
end   


% Initialize an array that will hold the decimal numbers we will upload
% to the arbitrary waveform generator.
if (continue == 1)
	samplesDec = zeros(noOfBytesNeeded, 1);
	for i = 1:noOfBytesNeeded
	   samplesDec(i) = bin2dec(samples1Dim(1, 8*i-7: i*8));
	end
end

	
% Replace 'output_file_name_including_full_path'
% with the full path and file name for the 
% output file holding the byte data that can
% be read by the Com-8001 module.
% For example:
% fid = fopen('c:\My Documents\Bin_Data_For_Com-8001.bin','w');
if (continue == 1)
	fid = fopen('output_file_name_including_full_path','w');
end


% Note that the file will be binary and thus most
% likely contain strange characters if you open it
% in a text editor such as notepad or wordpad. If 
% you prefer a decimal output file then you can copy 
% and paste the content of the samplesDec variable 
% into a file.
if (continue == 1)
	fwrite(fid, samplesDec, 'uint8');
   fclose(fid);
end


% If there are partial samples at the end that was not 
% saved or the number of bytes is not a valid window
% length in the Com-8001 module, then alert the user.
if (continue == 1)
	noOfBitsTotal = length(samples1Dim);
	if (round (noOfBitsTotal/256) ~=  (noOfBitsTotal/256))
	   disp(' ')
	   disp(' ')
	   disp('Warning!!!')
	   disp('The upload window length in the arbitrary waveform')
	   disp('generator must be a multiple of 256 bits (32 bytes). The')
	   disp('file you selected does not contain the right number of')
      disp('samples to meet this requirement. Partial samples at the')
	   disp('end will be ignored which can cause the waveform to have')
	   disp('discontinuities if the waveform is run in continuous')
      disp('download mode. If you wish to fix this problem then')
	   disp('make sure the total number of I and Q samples times')
	   disp('20/256 equals an integer value. A file was created')
	   disp('anyway.')
	   disp(' ')
	   disp(' ')
	end
end