% MATLAB program to convert a file containing I 
% and Q samples into a binary file that is accepted
% by the Com-8001 ComBlock module. This program will
% only work with the Com-4004 module and the only
% acceptable output width option is 14 bits.
% The input file must have two columns separated 
% by a tab. The first column must contain the I 
% samples in the range [-8,192  8,191] and the 
% second column must contain the Q samples in 
% the same range. The format for the samples is 
% regular readable ASCII (i.e. double, binary
% or anything else will not work).


% Replace 'input_file_name_including_full_path'
% with the full path and file name for the 
% I and the Q samples are stored.
% For example:
% iAndQSamples = load('c:\My Documents\iAndQData.txt');
iAndQSamples = load('waveform.dat');  


% Split the read matrix up into two column vector, 
% the first one holding the I data and the second 
% holding the Q data.
iSamples = iAndQSamples(:, 1);
qSamples = iAndQSamples(:, 2);


% Find out how many samples there are. The number of I
% samples is the same as the number of Q samples.
noOfISamples = length(iSamples);

% Make sure all the samples are in the 
% valid range, [-8,192  8,191].
contd = 1;
if (max(iSamples) > 8191) 
  disp('Too big I value in input file.')
  disp('No file will be created.')
    contd = 0;
elseif (min(iSamples) < -8192) 
disp('Too small I value in input file.')   
  disp('No file will be created.')
    contd = 0;
end
if (max(qSamples) > 8191) 
disp('Too big Q value in input file.')   
  disp('No file will be created.')
    contd = 0;
elseif (min(qSamples) < -8192) 
disp('Too small Q value in input file.')   
  disp('No file will be created.')
    contd = 0;
end

% convert to uint16 for easier bit manipulation
% add +8192 to make samples positive (uint16 need it)
% then flip the MSb to go back to 2's complement representation
ius = bitxor(uint16(iSamples + 8192),8192);
qus = bitxor(uint16(qSamples + 8192),8192);

% Find out how many bits we can convert into bytes. If the total number
% of bits is not a multiple of 8 then we will not be able to include
% all bits.
if (contd == 1)
    noOfBytesNeeded = ceil(noOfISamples*28/8);
end	


% Initialize an array that will hold the decimal numbers we will upload
% to the arbitrary waveform generator.
if (contd == 1)
	samplesDec = zeros(noOfBytesNeeded, 1);
    j = 1;
	for i = 1:2:noOfISamples
        samplesDec(j) =  bitand(bitshift(ius(i),-6),255);  % i(13:6)
        b = bitand(bitshift(ius(i),2),252); %i(5:0)
        c = bitand(bitshift(qus(i),-12),3); %q(13:12)
        samplesDec(j+1) = bitor(b,c);
        samplesDec(j+2) = bitand(bitshift(qus(i),-4),255); %q(11:4)
        b = bitand(bitshift(qus(i),4),240); %q(3:0)
        c = bitand(bitshift(ius(i+1),-10),15); %i(13:10)
        samplesDec(j+3) = bitor(b,c);
        samplesDec(j+4) = bitand(bitshift(ius(i+1),-2),255); %i(9:2)
        b = bitand(bitshift(ius(i+1),6),192); %i(1:0)
        c = bitand(bitshift(qus(i+1),-8),63); %q(13:8)
        samplesDec(j+5) = bitor(b,c);
        samplesDec(j+6) = bitand(qus(i+1),255); %q(7:0)
       j = j + 7;
	end
end

	
% Replace 'output_file_name_including_full_path'
% with the full path and file name for the 
% output file holding the byte data that can
% be read by the Com-8001 module.
% For example:
% fid = fopen('c:\My Documents\Bin_Data_For_Com-8001.bin','w');
if (contd == 1)
	fid = fopen('waveform.bin','w');
end	


% Note that the file will be binary and thus most
% likely contain strange characters if you open it
% in a text editor such as notepad or wordpad. If 
% you prefer a decimal output file then you can copy 
% and paste the content of the samplesDec variable 
% into a file.
if (contd == 1)
	fwrite(fid, samplesDec, 'uint8');
   fclose(fid);
end   


% If there are partial samples at the end that was not 
% saved or the number of bytes is not a valid window
% length in the Com-8001 module, then alert the user.
if (contd == 1)
	if (round (noOfBytesNeeded/32) ~=  (noOfBytesNeeded/32))
	   disp(' ')
	   disp(' ')
	   disp('Warning!!!')
	   disp('The upload window length in the arbitrary waveform')
	   disp('generator must be a multiple of 256 bits (32 bytes). The')
	   disp('file you selected does not contain the right number of')
      disp('samples to meet this requirement. Partial samples at the')
	   disp('end will be ignored which can cause the waveform to have')
	   disp('discontinuities if the waveform is run in continuous')
      disp('download mode. If you wish to fix this problem then')
	   disp('make sure the total number of I and Q samples times')
	   disp('14/256 equals an integer value. A file was created')
	   disp('anyway.')
	   disp(' ')
	   disp(' ')
	end
end